"""FlashXTest library to interface with backend.FlashTest"""

import os

from .. import backend

def setup(testDir,source,site):
    """
    Setup configuration

    Arguments
    ---------
    testDir : Test directory
    source  : Source directory
    site    : Site directory
    """
    configTemplate = os.path.dirname(backend.FlashTest.__file__)+'/configTemplate'
    configFile = str(testDir)+'/Config'

    cfile =  open(configFile, 'w') 
    with open(configTemplate) as ctemplate:
       lines = ctemplate.readlines()
       for line in lines:
            if source: 
                line = line.replace("pathToFlash:", str("pathToFlash:        "+source))
                line = line.replace("pathToOutdir:", str("pathToOutdir:       "+testDir+"/TestResults"))
            if site:
                line = line.replace("flashSite:", str("flashSite:          "+site))
            cfile.write(line)
    cfile.close()

    print("Initialized FlashXTest Configuration")

def run(testDir,testDict,mainDict):
    """
    Run flashTest.py from backend/FlashTest

    Arguments:
    ----------
    Arguments:
    testDir   : Test directory
    testDict  : Test dictionary
    mainDict  : Main dictionary
    """
    # Environment variable for OpenMP
    os.environ['OMP_NUM_THREADS'] = str(1)

    # Build sfocu
    __buildSFOCU(mainDict)

    # Cache the value of Outdir from Config file
    pathToOutdir = mainDict['pathToOutdir']

    # Create output directory if it does on exist
    os.system('mkdir -pv {0}'.format(pathToOutdir))

    mainDict['configFile'] = __getConfig(testDir)

    for testName,testList in testDict.items():
        
        mainDict['pathToInfo'] = __getTestInfo(testDir,testName,mainDict['flashSite'])
        mainDict['pathToOutdir'] = pathToOutdir+'/'+str(testName)

        # run flashTest.py and with desired configuration
        #
        exit_status = os.system('python3 {0}/flashTest.py \
                                 -z {1} \
                                 -o {2} \
                                 -c {3} \
                                 -i {4} \
                                 -v \
                                 -L \
                                 -s {5} \
                                 {6}'.format(os.path.dirname(backend.FlashTest.__file__),
                                             mainDict['pathToFlash'],
                                             mainDict['pathToOutdir'],
                                             mainDict['configFile'],
                                             mainDict['pathToInfo'],
                                             mainDict['flashSite'],
                                             " ".join(testList)))

    os.system('rm {0} {1}'.format(mainDict['pathToInfo'],mainDict['configFile']))

    # Handle errors
    # TODO: this is not working need to fix
    if exit_status != 0:
        print('FlashTest returned exit status {0}'.format(exit_status))
        print('---------------------------------------------------------')
        raise ValueError

    else:
        print('FlashTest reports SUCCESS')

def parseFile(configFile,**attributes):
    """
    parse Config file to get flashDir and site info

    Arguments
    --------
    configFile  : Path to config file for test
    attributes  : Dictionary to override values from Config file

    Returns
    -------
    mainDict: Dictionary for keys in the config file
    """
    mainDict=backend.FlashTest.lib.parseFile(configFile)

    for key,value in attributes.items(): 
        if value and key in mainDict:
            mainDict[key] = value

    return mainDict

def __buildSFOCU(mainDict):
    """
    Build SFOCU (Serial Flash Output Comparison Utility)

    Arguments:
    ----------
    mainDict: Dictionary from Config file
    """
    # Build brand new version of sfocu
    os.chdir('{0}/tools/sfocu'.format(mainDict['pathToFlash']))
    os.system('make SITE={0} NO_NCDF=True sfocu clean'.format(mainDict['flashSite']))
    os.system('make SITE={0} NO_NCDF=True sfocu'.format(mainDict['flashSite']))

def __getTestInfo(testDir,testName,flashSite):
    """
    Get test info site

    Arguments:
    -----------
    testDir   : Test directory
    testName  : Name of test
    flashSite : Flash-X site

    Return:
    -------
    testSiteInfo : Site xml file
    """
    os.system('mkdir -pv {0}'.format(str(testDir)+'/.fxt'))

    testInfo = str(testDir)+'/{0}.xml'.format(testName)
    testSiteInfo = str(testDir)+'/.fxt'+'/test.info'.format(testName)

    with open('{0}'.format(testSiteInfo), 'w') as testFile: testFile.write('<{0}>\n'.format(flashSite))
    os.system('cat {0} >> {1}'.format(testInfo,testSiteInfo))
    with open('{0}'.format(testSiteInfo), 'a') as testFile: testFile.write('</{0}>\n'.format(flashSite))

    return testSiteInfo

def __getConfig(testDir):
    """
    Get config file for test

    Arguments:
    -----------
    testDir   : Test directory

    Return:
    -------
    configFile : Test Config file
    """
    os.system('mkdir -pv {0}'.format(str(testDir)+'/.fxt'))

    configFile = str(testDir)+'/Config'
    configTest = str(testDir)+'/.fxt'+'/Config'
    configMain = os.path.dirname(backend.FlashTest.__file__)+'/configMain'

    os.system('cat {0} {1} > {2}'.format(configFile,configMain,configTest))

    return configTest
